library(randomForest)

set.seed(616)

cat("Чтение обучающей и тестовой выборок\n")
train <- read.csv("../input/train.csv")
test  <- read.csv("../input/test.csv")
store <- read.csv("../input/store.csv")

cat("Что же у нас есть?\n")
head(train)
head(test)
head(store)

train <- merge(train, store)
test <- merge(test, store)

cat("Имена переменных\n")
head(train)
head(test)

###############################################################################

cat("Если магазин закрыт...\n")
table(train$Sales[train$Open==0])
sum(test$Open==0)
sum(test$Open==0, na.rm = T)
table(test$Open)
table(test$Open, useNA = "always")
test[is.na(test$Open),]
table(train$Open, useNA = "always")

cat("Оставляем в тренировочной выборке только объекты, когда магазин был открыт\n")
train <- train[ which(train$Open=='1'),]

###############################################################################

cat("Даты \n")
table(train$Date)
train$month <- as.integer(format(as.Date(train$Date), "%m"))
train$year <- as.integer(format(as.Date(train$Date), "%y"))
train$day <- as.integer(format(as.Date(train$Date), "%d"))
head(train)

test$month <- as.integer(format(as.Date(test$Date), "%m"))
test$year <- as.integer(format(as.Date(test$Date), "%y"))
test$day <- as.integer(format(as.Date(test$Date), "%d"))
head(test)

sort(unique(test$Date))
print(sort(unique(test$Date)))

cat("Что же получилось в итоге?")
summary(train)
plot(table(train$Sales))

###############################################################################

cat("Визуализация")
library(forecast)
agg<-aggregate(Sales ~ ., data=train[, c("Sales", "month" ,"year")], FUN=sum)
ts<-ts(agg$Sales/1000000, start=2013, frequency=12)

col = rainbow(3)
seasonplot(ts, col=col, year.labels.left = TRUE, pch=19, las=1, main = "Продажи по годам", ylab = "Суммарные продажи(млн)")

meanAnnual<-aggregate(Sales ~ ., data=agg[, c("Sales" ,"year")], FUN=mean)[-3,]
abline(h=meanAnnual[1,2]/1000000, col=col[1], lty=2)
abline(h=meanAnnual[2,2]/1000000, col=col[2], lty=2)
abline(v=7, col="grey50", lty=2)

########################################

cat("Визуализация")
library(forecast)
agg<-aggregate(Sales ~ ., data=train[train$Store==1, c("Sales", "month" ,"year")], FUN=sum)
ts<-ts(agg$Sales/1000000, start=2013, frequency=12)

col = rainbow(3)
seasonplot(ts, col=col, year.labels.left = TRUE, pch=19, las=1, main = "Продажи по годам (Store 1)", ylab = "Продажи")

meanAnnual<-aggregate(Sales ~ ., data=agg[, c("Sales" ,"year")], FUN=mean)[-3,]
abline(h=meanAnnual[1,2]/1000000, col=col[1], lty=2)
abline(h=meanAnnual[2,2]/1000000, col=col[2], lty=2)
abline(v=7, col="grey50", lty=2)

########################################

plotStore <- function(dt, store) { 
  dt <- subset(dt, dt$Store == store)
  
  dt$Date = as.Date(dt$Date, format = "%Y-%m-%d")
  dt <- dt[order(dt$Date),]
  
  
  plot(dt$Sales ~ dt$Date, 
       col = dt$DayOfWeek,
       pch = (as.integer(dt$StateHoliday)) + 19, 
       main = paste("store", store, "  n =", nrow(dt), "  ", "Assortment = ", dt$Assortment[1], ", StoreType = ",  dt$StoreType[1]))
  legend("topleft", legend=levels(factor(dt$DayOfWeek)), text.col=seq_along(levels(factor(dt$DayOfWeek))))
  return
}

sapply(sample(length(unique(train$Store)), 20), function(store) {
  # sapply(1:length(unique(train$Store)), function(store) {
  plotStore(train, store)
})

table(train$DayOfWeek)

###############################################################################
cat("Удаляем лишнее\n")

train$Date <- NULL
test$Date <- NULL

train$Customers <- NULL


###############################################################################

cat("Пытаемся обучить RF")
clf <- randomForest(train[,-c(4)], train$Sales, ntree=10, do.trace=TRUE)
cat("С первого раза не получилось...\n")
colSums(is.na(train))
train[is.na(train)] <- -99999
test[is.na(test)] <- -99999

head(train,1)
samp <- sample(1:nrow(train), 10000)
clf <- randomForest(train[samp,-c(3)], train$Sales[samp], ntree=10, do.trace=TRUE)
cat("Feature Importance\n")
varImpPlot(clf)
cat("Легко дообучить\n")
clf <- grow(clf, 10)
print(clf)

###############################################################################

val <- train[train$year ==  15 & train$month >= 6,]
train <- train[train$year ==  15 & train$month < 6,]

head(train,1)
head(val,1)

samp <- sample(1:nrow(train), 20000)
clf <- randomForest(train[samp,-c(3)], train$Sales[samp], ntree=10, do.trace=TRUE)
pred <- predict(clf, val[,-c(3)])

rmspe <- function(actual, pred, lev = NULL, model = NULL) {
  cond <- actual > 0
  out <- sqrt(mean(((actual[cond]-pred[cond])/actual[cond])^2))
  names(out) <- "RMSPE"
  out
}

cat("Первый результат:")
rmspe(val$Sales,pred)
plot(clf)

###############################################################################

library(caret)
set.seed(825)

rmspe <- function(data, lev = NULL, model = NULL) {
 cond <- data$obs > 0
 out <- sqrt(mean(((data$obs[cond]-data$pred[cond])/data$obs[cond])^2))
 names(out) <- "RMSPE"
 out
}

fitControl <- trainControl(
  ## 2-fold CV
  method = "repeatedcv",
  number = 2,
  ## repeated 2 times
  repeats = 2,
  summaryFunction=rmspe)

rfFit1 <- train(Sales ~ ., data = train[samp,],
                 method = "rf",  ntree=10,
                 trControl = fitControl,
                 metric = "RMSPE",
                 maximize = F,
                 verbose = T)
rfFit1

trellis.par.set(caretTheme())
ggplot(rfFit1)


fitControl <- trainControl(method = "none")

set.seed(825)
rfFit2 <- train(Sales ~ ., data = train[samp,],
                 method = "rf", ntree=10,
                 trControl = fitControl,
                 verbose = FALSE,
                 ## Only a single model can be passed to the
                 ## function when no resampling is used:
                 tuneGrid = data.frame(mtry=25),
                 metric = "RMSPE")
rfFit2

predict(rfFit2, newdata = head(val))

pred <- predict(rfFit2, newdata = test[,-c(2)])

submission <- data.frame(Id=test$Id, Sales=pred)
cat("saving the submission file\n")
write.csv(submission, "rf1.csv")

###############################################################################

fitControl <- trainControl(
  ## 2-fold CV
  method = "repeatedcv",
  number = 2,
  ## repeated 2 times
  repeats = 2,
  summaryFunction=rmspe)

gbmGrid <-  expand.grid(interaction.depth = c(1, 3, 5),
                        n.trees = (1:10)*50,
                        shrinkage = c(0.1, 0.15),
                        n.minobsinnode = 20)

gbmFit1 <- train(Sales ~ ., data = train[samp,],
                method = "gbm", 
                trControl = fitControl,
                metric = "RMSPE",
                maximize = F,
                verbose = T,
                tuneGrid = gbmGrid)
gbmFit1

trellis.par.set(caretTheme())
ggplot(gbmFit1)

###############################################################################
resamps <- resamples(list(GBM = gbmFit1,
                          RF = rfFit1))
resamps
summary(resamps)
